__title__ = "jusho"
__description__ = "Easy wrapper for the zip code data of Japan"
__url__ = "https://github.com/nagataaaas/Jusho"
__version_info__ = ("1", "0", "1")
__version__ = ".".join(__version_info__)
__author__ = "Yamato Nagata"
__author_email__ = "chickenwingswillfly@gmail.com"
__maintainer__ = "Yamato Nagata"
__license__ = "MIT"
__copyright__ = "(c) 2022 by Yamato Nagata"
